<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math"
    xmlns:map="http://www.w3.org/2005/xpath-functions/map"
    xmlns:doc="http://www.saxonica.com/ns/doc" xmlns:f="MyFunctions"
    xmlns:svg="http://www.w3.org/2000/svg" xmlns:saxon="http://saxon.sf.net/"
    exclude-result-prefixes="xs math f doc" version="3.0">

    <doc:synopsis title="Common barchart code">
        <p>Common code for use in template reporting via barcharts</p>
    </doc:synopsis>
    <xsl:variable name="dummy" as="element()">
        <dummy/>
    </xsl:variable>
    <!--<xsl:variable name="namespace.saxon" as="xs:string">http://saxon.sf.net/</xsl:variable>
    <xsl:variable name="namespace.XSLT2" as="xs:string"
        >http://www.w3.org/1999/XSL/Transform</xsl:variable>-->
    <xsl:variable name="namespaces" as="map(*)"
        select="map:merge(for $prefix in ('xsl','saxon')
        return map{$prefix : namespace-uri-for-prefix($prefix,$dummy)})"/>
    <xsl:variable name="match.namespaces" as="xs:string" doc:doc="A regex that should match certain namespace declarations"
        select="
            string-join(
            map:for-each($namespaces, function ($prefix, $uri) {
                'xmlns:' || $prefix || '=&quot;' || $uri || '&quot;\s+'
            })
            , '|')"/>

    <xsl:mode name="stats.barchart" on-no-match="fail"/>
    <xsl:template match="group" mode="stats.barchart">
        <xsl:param name="name" as="xs:string" required="true" tunnel="yes"/>
        <xsl:param name="valueOfInterest" as="function(*)" required="true" tunnel="yes"
            doc:doc="A function $valueOfInterest(rule/case)that will extract the value of interest for this particular graph"/>
        <xsl:param name="svg.root" select="@svg" tunnel="yes"/>
        <xsl:param name="sources.matches" as="map(*)" required="true" tunnel="yes"/>
        <xsl:param name="sources.templates" as="map(*)" required="true" tunnel="yes"/>
        <xsl:param name="options" as="map(*)" tunnel="yes"/>
        <xsl:variable name="n.cases" select="count(case)"/>

        <!--<xsl:variable name="modes"
            select="
                map:merge(stylesheet/mode/map:entry(@name, template))"/>
        <xsl:variable name="sources"
            select="
                distinct-values(map:for-each($modes, function ($k, $v) {
                    $v/@module
                }))"
            as="xs:string*"/>-->
        <!--<xsl:variable name="stylesheet.templates"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], doc(.)//xsl:template[@match]))"/>
        <xsl:variable name="sources.short"
            select="distinct-values($sources ! tokenize(., '/')[last()])"/>
        <xsl:variable name="short2long"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], .))"/>-->
        <!--<xsl:variable name="sources.matches"
            select="map:merge($sources ! map:entry(., map:merge(doc(.)//xsl:template/@match/map:entry(saxon:line-number(.), .))))"/>-->

        <xsl:variable name="key" as="element()*" select="svg:makeKey(case)"/>
        <!--<xsl:variable name="valueOfInterest" as="function(*)"
            select="
            function ($c) {
            $c/@times.total
            }"/>-->
        <xsl:variable name="values" as="element()*"
            select="f:values-of-interest(., $valueOfInterest, $sources.matches)"/>

        <xsl:for-each select="$values[@isBig = true()]">
            <xsl:variable name="displayed" select="reverse(subsequence(./*, 1, 7))"/>
            <xsl:variable name="graphics" as="element()*">
                <xsl:apply-templates select="$displayed" mode="svg:barchart">
                    <xsl:with-param name="sep" select="$n.cases + 2" tunnel="yes"/>
                    <xsl:with-param name="data" as="function(*)" select="$valueOfInterest"
                        tunnel="yes"/>
                </xsl:apply-templates>
            </xsl:variable>
            <xsl:variable name="graphics.labels" as="element()*">
                <xsl:apply-templates select="$displayed" mode="svg:barchart.label">
                    <xsl:with-param name="sep" select="$n.cases + 2" tunnel="yes"/>
                    <xsl:with-param name="other.attributes" as="attribute()*" tunnel="yes">
                        <xsl:attribute name="x" select="$width div 4 - 20"/>
                        <xsl:attribute name="text-anchor">end</xsl:attribute>
                    </xsl:with-param>
                </xsl:apply-templates>
            </xsl:variable>
            <xsl:variable name="graphics.labels" as="element()*"
                select="svg:scale($graphics.labels,
                map:merge( (f:graphics.vals($graphics.labels,map {'x' : 0, 'y' : 0, 'width' : $width div 4,'height' : $height}),
                map {'scale.x' : 1})))"/>

            <xsl:variable name="g"
                select="map:merge((f:graphics.vals($graphics,map {'x' : $width div 4, 'y' : 0, 'width': 3 * $width div 4,'height':$height}), 
                $options,
                map{ 
                'grid' : svg:makeGrid.vertical#3,
               (: 'axis.x.format' : function($a) {format-number($a idiv 1000000,'0')} ,
                'axis.x.label' : 'Total / ms',:)
                'ABCDEF' : $key,                
                'label' : ($options('label'),'mode:'||@name)
                } ))"/>
            <xsl:call-template name="svg:makeFile" use-when="true()">
                <xsl:with-param name="svg.root" select="$svg.root"/>
                <xsl:with-param name="mode" select="@name"/>
                <xsl:with-param name="g" select="$g"/>
                <xsl:with-param name="name" select="$name"/>
                <xsl:with-param name="content" select="$graphics"/>
                <xsl:with-param name="content.unscaled" select="$graphics.labels"/>
            </xsl:call-template>
        </xsl:for-each>

    </xsl:template>

    <xsl:template match="group" mode="stats.barchart.svg">
        <xsl:param name="name" as="xs:string" required="true" tunnel="yes"/>
        <xsl:param name="valueOfInterest" as="function(*)" required="true" tunnel="yes"/>
        <xsl:param name="svg.root" select="@svg" tunnel="yes"/>
        <xsl:param name="sources.matches" as="map(*)" required="true" tunnel="yes"/>
        <xsl:param name="sources.templates" as="map(*)" required="true" tunnel="yes"/>
        <xsl:param name="options" as="map(*)" tunnel="yes"/>
        <xsl:variable name="n.cases" select="count(case)"/>

        <!--<xsl:variable name="modes"
            select="
                map:merge(stylesheet/mode/map:entry(@name, template))"/>
        <xsl:variable name="sources"
            select="
                distinct-values(map:for-each($modes, function ($k, $v) {
                    $v/@module
                }))"
            as="xs:string*"/>-->
        <!--<xsl:variable name="stylesheet.templates"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], doc(.)//xsl:template[@match]))"/>
        <xsl:variable name="sources.short"
            select="distinct-values($sources ! tokenize(., '/')[last()])"/>
        <xsl:variable name="short2long"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], .))"/>-->
        <!--<xsl:variable name="sources.matches"
            select="map:merge($sources ! map:entry(., map:merge(doc(.)//xsl:template/@match/map:entry(saxon:line-number(.), .))))"/>-->

        <xsl:variable name="key" as="element()*" select="svg:makeKey(case)"/>
        <!--<xsl:variable name="valueOfInterest" as="function(*)"
            select="
            function ($c) {
            $c/@times.total
            }"/>-->
        <xsl:variable name="values" as="element()*"
            select="f:values-of-interest(., $valueOfInterest, $sources.matches)"/>

        <xsl:for-each select="$values[@isBig = true()]">
            <xsl:variable name="displayed" select="reverse(subsequence(./*, 1, 7))"/>
            <xsl:variable name="graphics" as="element()*">
                <xsl:apply-templates select="$displayed" mode="svg:barchart">
                    <xsl:with-param name="sep" select="$n.cases + 2" tunnel="yes"/>
                    <xsl:with-param name="data" as="function(*)" select="$valueOfInterest"
                        tunnel="yes"/>
                </xsl:apply-templates>
            </xsl:variable>
            <xsl:variable name="graphics.labels" as="element()*">
                <xsl:apply-templates select="$displayed" mode="svg:barchart.label">
                    <xsl:with-param name="sep" select="$n.cases + 2" tunnel="yes"/>
                    <xsl:with-param name="other.attributes" as="attribute()*" tunnel="yes">
                        <xsl:attribute name="x" select="$width div 4 - 20"/>
                        <xsl:attribute name="text-anchor">end</xsl:attribute>
                    </xsl:with-param>
                </xsl:apply-templates>
            </xsl:variable>
            <xsl:variable name="graphics.labels" as="element()*"
                select="svg:scale($graphics.labels,
                map:merge( (f:graphics.vals($graphics.labels,map {'x' : 0, 'y' : 0, 'width' : $width div 4,'height' : $height}),
                map {'scale.x' : 1})))"/>

            <xsl:variable name="g"
                select="map:merge((f:graphics.vals($graphics,map {'x' : $width div 4, 'y' : 0, 'width': 3 * $width div 4,'height':$height}), 
                $options,
                map{ 
                'grid' : svg:makeGrid.vertical#3,
                (: 'axis.x.format' : function($a) {format-number($a idiv 1000000,'0')} ,
                'axis.x.label' : 'Total / ms',:)
                'ABCDEF' : $key,                
                'label' : ($options('label'),'mode:'||@name)
                } ))"/>
            <xsl:call-template name="svg:makeSVG">
                <xsl:with-param name="svg.root" select="$svg.root"/>
                <xsl:with-param name="mode" select="@name"/>
                <xsl:with-param name="g" select="$g"/>
                <xsl:with-param name="name" select="$name"/>
                <xsl:with-param name="content" select="$graphics"/>
                <xsl:with-param name="content.unscaled" select="$graphics.labels"/>
            </xsl:call-template>
        </xsl:for-each>

    </xsl:template>
    <xsl:variable name="serialize.map"
        select="map{QName('http://saxon.sf.net/','line-length') : 40,
            QName('http://saxon.sf.net/','indent-spaces') : 1,
            'indent' : true()}"/>
    <xsl:function name="f:serialize-template" as="xs:string">
        <xsl:param name="template" as="element()"/>
        <xsl:variable name="template" as="element()?">
            <xsl:apply-templates select="$template" mode="remove-whitespace"/>
        </xsl:variable>
        <xsl:value-of
            select="serialize($template, $serialize.map) ! replace(., $match.namespaces, '')"/>
    </xsl:function>

    <xsl:mode name="svg:barchart.label" on-no-match="deep-skip"/>
    <xsl:template match="rule" mode="svg:barchart.label" xmlns="http://www.w3.org/2000/svg"
        expand-text="yes">
        <xsl:param name="sep" as="xs:integer?" select="4" tunnel="yes"/>
        <xsl:param name="other.attributes" as="attribute()*" select="()" tunnel="yes"/>
        <xsl:param name="sources.templates" as="map(*)" required="true" tunnel="yes"/>
        <xsl:variable name="template"
            select="if(exists(@module)) then $sources.templates(@module)(number(@line)) else ()"/>
        <!--<xsl:if test="exists(@module)">
            <xsl:message>
                <xsl:call-template name="map:print">
                    <xsl:with-param name="items" select="$sources.templates(@module)"/>
                </xsl:call-template>
            </xsl:message>
        </xsl:if>-->
        <!--<xsl:variable name="template" as="element()?">
            <xsl:apply-templates select="$template" mode="remove-whitespace"/>
        </xsl:variable>
        <xsl:variable name="template.serialized"
            select="serialize($template, $serialize.map) ! replace(., $remove.namespaces, '')"/>-->
        <!--  <xsl:message select="'TEMPLATE:', $template.serialized"/>-->
        <xsl:variable name="pos" select="(position() - 1) * $sep"/>
        <text class="match" x="0" y="{$pos + $sep - 1.5}">
            <xsl:if test="exists($template)">
                <xsl:attribute name="class">match hasTip</xsl:attribute>
                <xsl:attribute name="tip">codeSample</xsl:attribute>
                <xsl:attribute name="tip.data" expand-text="yes"
                    >{tokenize(@module,'/')[last()]}#{@line} rank:{@rank}
                    sequence:{@seq}</xsl:attribute>
                <xsl:attribute name="tip.pre" select="f:serialize-template($template)"/>
            </xsl:if>
            <xsl:sequence select="$other.attributes"/>{($template/@match,@match)[1]}</text>
    </xsl:template>


    <xsl:mode name="remove-whitespace" on-no-match="shallow-copy"/>
    <xsl:template match="*" mode="remove-whitespace">
        <xsl:copy copy-namespaces="no">
            <xsl:apply-templates select="@*, node()" mode="#current"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="text()[not(matches(., '\S'))]" mode="remove-whitespace"/>

    <xsl:mode name="no-namespaces" on-no-match="shallow-copy"/>
    <xsl:template match="*" mode="no-namespaces"/>
    <!--  <xsl:mode name="svg:barchart-call" on-no-match="deep-skip"/>-->
    <xsl:mode name="svg:barchart" on-no-match="deep-skip"/>

    <xsl:template match="rule" mode="svg:barchart" xmlns="http://www.w3.org/2000/svg"
        expand-text="yes">
        <xsl:param name="sep" as="xs:integer?" select="4" tunnel="yes"/>
        <xsl:variable name="pos" select="(position() - 1) * $sep"/>
        <!--       <text class="match" x="0.5" y="{$pos + $sep - 1.5}">{@order}:{@rank} → {@match}</text>-->
        <xsl:apply-templates mode="#current">
            <xsl:with-param name="y" select="$pos + 1" tunnel="yes"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="rule/case" mode="svg:barchart" xmlns="http://www.w3.org/2000/svg">
        <xsl:param name="data" as="function(*)" required="true" tunnel="yes"/>
        <xsl:param name="data.format" as="function(*)?"
            select="
                function ($d) {
                    $d
                }"
            tunnel="yes"/>
        <xsl:param name="atts" as="attribute()*" tunnel="yes"/>
        <xsl:param name="y" as="xs:integer?" select="0" tunnel="yes"/>
        <xsl:variable name="start" as="xs:integer" select="0"/>
        <xsl:variable name="end" as="xs:integer" select="xs:integer(number($data(.)))"/>
        <xsl:variable name="y" select="position() + $y - 1"/>
        <line class="{@name} total hasTip" tip="value" tip.data="{$data.format($data(.))}" x1="0"
            x2="{$end}" y1="{$y}" y2="{$y}"/>
    </xsl:template>





    <xsl:template match="group" mode="stats.table" xmlns="" expand-text="yes">
        <xsl:param name="name" as="xs:string" required="true" tunnel="yes"/>
        <xsl:param name="valueOfInterest" as="function(*)" required="true" tunnel="yes"/>
        <xsl:param name="svg.root" select="@svg" tunnel="yes"/>
        <xsl:param name="sources.matches" as="map(*)" required="true" tunnel="yes"/>
        <xsl:param name="options" as="map(*)" select="map {}" tunnel="yes"/>
        <xsl:variable name="n.cases" select="count(case)"/>
        <xsl:variable name="values" as="element()*"
            select="f:values-of-interest(., $valueOfInterest, $sources.matches)"/>
        <table name="{$name}">
            <thead>
                <tr>
                    <th class="sortable" rowspan="2">Mode</th>
                    <th class="sortable" rowspan="2">Location</th>
                    <th class="sortable" rowspan="2">Rank</th>
                    <th class="sortable" rowspan="2">Seq.</th>
                    <th class="sortable" rowspan="2">Match</th>
                    <th colspan="{count(case)}">Times/ms</th>
                </tr>
                <tr>
                    <xsl:for-each select="case">
                        <th class="sortable">{@label}</th>
                    </xsl:for-each>
                </tr>
            </thead>
            <tbody class="stripe">
                <xsl:for-each select="$values[@isBig = true()]">
                    <xsl:variable name="displayed" select="subsequence(./*, 1, 7)"/>
                    <xsl:apply-templates select="$displayed" mode="table">
                        <xsl:with-param name="data" as="function(*)" select="$valueOfInterest"
                            tunnel="yes"/>
                    </xsl:apply-templates>
                </xsl:for-each>
            </tbody>
        </table>
    </xsl:template>
    <xsl:mode name="table" on-no-match="deep-skip"/>
    <xsl:template match="rule" mode="table" xmlns="" expand-text="yes">
        <xsl:param name="sources.templates" as="map(*)" required="true" tunnel="yes"/>
        <xsl:variable name="template"
            select="if(exists(@module)) then $sources.templates(@module)(number(@line)) else ()"/>
        <tr>
            <td>{../@name}</td>
            <td>
                <xsl:if test="exists($template)">
                    <xsl:attribute name="class">match hasTip</xsl:attribute>
                    <xsl:attribute name="tip">codeSample</xsl:attribute>
                    <xsl:attribute name="tip.data" expand-text="yes"
                        >{tokenize(@module,'/')[last()]}#{@line} rank:{@rank}
                        sequence:{@seq}</xsl:attribute>
                    <xsl:attribute name="tip.pre" select="f:serialize-template($template)"/>
                </xsl:if>{($built-in.mode-defaults(@loc),(tokenize(@loc,'/')[last()]))[1]}</td>
            <td>{@rank}</td>
            <td>{@seq}</td>
            <td>{@match}</td>
            <xsl:apply-templates mode="#current"/>
        </tr>
    </xsl:template>
    <xsl:template match="rule/case" mode="table" xmlns="" expand-text="yes">
        <xsl:param name="data" as="function(*)" required="true" tunnel="yes"/>
        <xsl:param name="data.format" as="function(*)?"
            select="
                function ($d) {
                    $d
                }"
            tunnel="yes"/>
        <td class="number">{$data.format($data(.))}</td>
    </xsl:template>
</xsl:stylesheet>
